import {
  select,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  scaleOrdinal,
  schemeTableau10,
  format,
} from 'd3';

export const viz = (
  svg,
  {
    data,
    xValue,
    xAxisLabelText,
    xAxisLabelOffset,
    yValue,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginTop,
    marginRight,
    marginBottom,
    width,
    height,
    circleRadius,
    circleOpacity,
  },
) => {
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([marginLeft, width - marginRight]);
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([height - marginBottom, marginTop]);

  const brandScale = scaleOrdinal()
    .domain(data.map((d) => d.Car_brand))
    .range(schemeTableau10);

  const tooltip = select('body')
    .selectAll('.tooltip')
    .data([null])
    .join('div')
    .attr('class', 'tooltip')
    .style('opacity', 0)
    .style('position', 'absolute')
    .style('padding', '10px')
    .style('background', 'white')
    .style('border', '1px solid lightgrey')
    .style('border-radius', '5px')
    .style('pointer-events', 'none')
    .style('font-family', 'Arial, sans-serif');

  svg
    .selectAll('circle')
    .data(data)
    .join('circle')
    .attr('cx', (d) => xScale(xValue(d)))
    .attr('cy', (d) => yScale(yValue(d)))
    .attr('r', circleRadius)
    .attr('stroke', 'black')
    .style('stroke-width', 2)
    .attr('opacity', circleOpacity)
    .attr('fill', (d) => brandScale(d.Car_brand))
    .attr(
      'class',
      (d) =>
        `brand-circle brand-${d.Car_brand.replace(/\s+/g, '-')}`,
    )
    .on('mouseover', (event, d) => {
      const [tooltipWidth, tooltipHeight] = [200, 60];
      let x = event.pageX + 10;
      let y = event.pageY - 28;

      if (x + tooltipWidth > window.innerWidth) {
        x = event.pageX - tooltipWidth - 10;
      }
      if (y + tooltipHeight > window.innerHeight) {
        y = event.pageY - tooltipHeight - 10;
      }

      tooltip
        .transition()
        .duration(200)
        .style('opacity', 0.9);
      tooltip
        .html(
          `Car name: ${d.Car_name}<br>Driving Range: ${format(',')(xValue(d))}<br>Price: ${format(',.2f')(yValue(d))}`,
        )
        .style('left', `${x}px`)
        .style('top', `${y}px`);
      select(event.currentTarget).attr(
        'r',
        circleRadius * 1.5,
      );
    })
    .on('mouseout', (event) => {
      tooltip
        .transition()
        .duration(500)
        .style('opacity', 0);
      select(event.currentTarget).attr('r', circleRadius);
    });

  svg
    .selectAll('.x-axis')
    .data([null])
    .join('g')
    .attr('class', 'x-axis')
    .attr(
      'transform',
      `translate(0,${height - marginBottom})`,
    )
    .call(axisBottom(xScale));

  svg
    .selectAll('.y-axis')
    .data([null])
    .join('g')
    .attr('class', 'y-axis')
    .attr('transform', `translate(${marginLeft},0)`)
    .call(axisLeft(yScale));

  svg
    .selectAll('.x-axis-label')
    .data([null])
    .join('text')
    .attr('class', 'x-axis-label')
    .attr(
      'transform',
      `translate(${width / 2},${height - xAxisLabelOffset})`,
    )
    .style('text-anchor', 'middle')
    .text(xAxisLabelText);

  svg
    .selectAll('.y-axis-label')
    .data([null])
    .join('text')
    .attr('class', 'y-axis-label')
    .attr('transform', 'rotate(-90)')
    .attr('y', yAxisLabelOffset)
    .attr('x', -(height / 2))
    .attr('dy', '1em')
    .style('text-anchor', 'middle')
    .text(yAxisLabelText);

  const legend = svg
    .selectAll('.legend')
    .data([null])
    .join('g')
    .attr('class', 'legend')
    .attr(
      'transform',
      `translate(${marginLeft + 60}, ${marginTop + 20})`,
    );

  legend
    .selectAll('.legend-title')
    .data([null])
    .join('text')
    .attr('class', 'legend-title')
    .attr('x', 0)
    .attr('y', -10)
    .style('font-size', '14px')
    .style('font-family', 'Arial, sans-serif')
    .style('text-decoration', 'underline')
    .text('Top 10 Car Brands');

  const legendItem = legend
    .selectAll('.legend-item')
    .data(brandScale.domain().slice(0, 10))
    .join('g')
    .attr('class', 'legend-item')
    .style('cursor', 'pointer')
    .attr('transform', (d, i) => `translate(0, ${i * 20})`)
    .on('click', (event, selectedBrand) => {
      const isSelectedBrand = (d) =>
        d.Car_brand === selectedBrand;

      svg
        .selectAll('.brand-circle')
        .transition()
        .duration(300)
        .attr('opacity', (d) =>
          isSelectedBrand(d) ? circleOpacity : 0.04,
        );
    });

  legendItem
    .append('rect')
    .attr('width', 18)
    .attr('height', 18)
    .attr('fill', brandScale);

  legendItem
    .append('text')
    .attr('x', 24)
    .attr('y', 9)
    .attr('dy', '0.35em')
    .style('font-size', '12px')
    .style('font-family', 'Arial, sans-serif')
    .text((d) => d);

  //ALL
  legend
    .selectAll('.reset-button')
    .data([null])
    .join('text')
    .attr('class', 'reset-button')
    .attr('x', 0)
    .attr('y', 213)
    .style('font-size', '13px')
    .style('font-family', 'Arial, sans-serif')
    .style('cursor', 'pointer')
    .text('RESET')
    .on('click', () => {
      svg
        .selectAll('.brand-circle')
        .transition()
        .duration(300)
        .attr('opacity', circleOpacity);
    });
};
